package Form::Factory::Control::Role::MultiLine;
$Form::Factory::Control::Role::MultiLine::VERSION = '0.022';
use Moose::Role;

# ABSTRACT: multiline controls


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Form::Factory::Control::Role::MultiLine - multiline controls

=head1 VERSION

version 0.022

=head1 DESCRIPTION

Marks controls that have a multiline text entry.

=head1 AUTHOR

Andrew Sterling Hanenkamp <hanenkamp@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by Qubling Software LLC.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
